/*
 * DatabasePlugin.java
 *
 * Created on July 13, 2005, 2:46 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package lu.tudor.santec.settings;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

import lu.tudor.santec.i18n.Relocalizable;
import lu.tudor.santec.i18n.Translatrix;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 *
 * @author nmac
 */

//***************************************************************************
//* Class Definition and Members                                            *
//***************************************************************************

public class DatabasePlugin extends SettingsPlugin implements Relocalizable,
                                                              ActionListener
    {
    private static Logger m_Logger = Logger.getLogger ("lu.tudor.santec.settings.DatabasePlugin");

    private JPanel          m_DriverPanel;
    private JLabel          m_DriverDescription;
    private JComboBox       m_DriverSelector;
    
    private JPanel          m_ConnectionPanel;
    private JLabel          m_ConnectionDescription;
    private JTextField      m_DSN;
    private JTextField      m_Username;
    private JPasswordField      m_Password;
    
    private JButton         m_TestConnection;
    
    private JLabel          m_DriverLabel;
    private JLabel          m_DSNLabel;
    private JLabel          m_UsernameLabel;
    private JLabel          m_PasswordLabel;
    private JLabel          m_StatusMessage;
    
    private Vector          m_DatabaseDrivers;
    private Hashtable       m_DriverLookup;
    private DatabaseDriver  m_DatabaseDriver;
    
    private int             m_ConnectionStatus;
    
//***************************************************************************
//* Constants                                                               *
//***************************************************************************

    public static final DatabaseDriver MYSQL = new DatabaseDriver ("com.mysql.jdbc.Driver",
                                                                     "MySQL",
                                                                     "jdbc:mysql:[//[host][:port]/]database");
    
    public static final DatabaseDriver POSTGRESQL = new DatabaseDriver ("org.postgresql.Driver",
                                                                          "PostgreSQL",
                                                                          "jdbc:postgresql:[//[host][:port]/]database");

    public static final DatabaseDriver HSQLDB = new DatabaseDriver ("org.hsqldb.jdbcDriver",
                                                                          "HSQL Server",
                                                                          "jdbc:hsqld:[//[host][:port]/]database");

    public static final DatabaseDriver ORACLE = new DatabaseDriver ("oracle.jdbc.driver.OracleDriver",
                                                                          "Oracle Database",
                                                                          "jdbc:oracle:thin:@host:port[:oraclesid]");

    public static final int c_None      =   0;
    public static final int c_Okay      =   1;
    public static final int c_Failure   =   2;
    
//***************************************************************************
//* Constructor                                                             *
//***************************************************************************

//---------------------------------------------------------------------------
/** Creates a new instance of DatabasePlugin                                */
//---------------------------------------------------------------------------

public DatabasePlugin(String p_Name) 
    {
    super (p_Name);
    
    FormLayout              l_Layout;
    PanelBuilder            l_Builder;
    CellConstraints         l_Constraints;
    
    m_DatabaseDrivers   = new Vector ();
    m_DriverLookup      = new Hashtable ();
    m_DatabaseDriver    = null;

     // Create Driver SubPanel

    l_Layout      = new FormLayout ("right:50dlu,5dlu,left:120dlu:grow","center:pref,5dlu,center:pref");
    l_Constraints = new CellConstraints();
    
    m_DriverPanel = createSubPanel (Translatrix.getTranslationString("database:DriverTitle"));
    m_DriverPanel.setLayout (l_Layout);
    
    l_Builder     = new PanelBuilder (l_Layout,m_DriverPanel);  

    m_DriverDescription = new JLabel ();
    m_DriverDescription.setText (Translatrix.getTranslationString("database:DriverDescription"));

    l_Builder.add (m_DriverDescription,l_Constraints.xyw (1,1,3,"fill, center"));

    m_DriverLabel = new JLabel (Translatrix.getTranslationString("database:DriverLabel"));
           
    m_DriverSelector = new JComboBox ();
    m_DriverSelector.addActionListener (this);
    
    l_Builder.add (m_DriverLabel,l_Constraints.xy (1,3));
    l_Builder.add (m_DriverSelector,l_Constraints.xy (3,3));

    addSubPanel (m_DriverPanel);

    // Create Connection SubPanel

    l_Layout      = new FormLayout ("right:50dlu,5dlu,left:120dlu:grow",
                                    "center:pref,5dlu,center:pref,5dlu,center:pref,5dlu,center:pref,5dlu,center:pref,5dlu,center:pref");
    l_Constraints = new CellConstraints();
      
    m_ConnectionPanel = createSubPanel (Translatrix.getTranslationString("database:ConnectionTitle"));
    m_ConnectionPanel.setLayout (l_Layout);
        
    l_Builder     = new PanelBuilder (l_Layout,m_ConnectionPanel);  

    // Add Connection Description to Connection Panel

    m_ConnectionDescription = new JLabel ();
    m_ConnectionDescription.setText (Translatrix.getTranslationString("database:Description"));
    
    l_Builder.add (m_ConnectionDescription,l_Constraints.xyw (1,1,3,"fill, center"));
    
    // Add Database DSN Control to Connection Panel
    
    m_DSNLabel = new JLabel (Translatrix.getTranslationString("database:DSNLabel"));
       
    m_DSN = new JTextField ("");
    m_DSN.addActionListener (this);
    
    l_Builder.add (m_DSNLabel,l_Constraints.xy (1,3));
    l_Builder.add (m_DSN,l_Constraints.xy (3,3,"fill, center"));
       
   // Add Database User Control to Connection Panel
     
    m_UsernameLabel = new JLabel (Translatrix.getTranslationString("database:UsernameLabel"));
      
    m_Username = new JTextField ("");
    
    l_Builder.add (m_UsernameLabel,l_Constraints.xy (1,5));
    l_Builder.add (m_Username,l_Constraints.xy (3,5,"fill, center"));

   // Add Database Password Control to Connection Panel
    
    m_PasswordLabel = new JLabel (Translatrix.getTranslationString("database:PasswordLabel"));
   
    m_Password = new JPasswordField ("");
     
    l_Builder.add (m_PasswordLabel,l_Constraints.xy (1,7));
    l_Builder.add (m_Password,l_Constraints.xy (3,7,"fill, center"));

   // Add Test Button to Connection Panel
       
    m_TestConnection = new JButton (Translatrix.getTranslationString("database:TestButton"));
    m_TestConnection.addActionListener (this);
    
    l_Builder.add (m_TestConnection,l_Constraints.xy (3,9));
        
    // Add Connection Status to Connection Panel
    
    m_ConnectionStatus = c_None;
    
    m_StatusMessage = new JLabel();
    m_StatusMessage.setVerticalTextPosition (JLabel.TOP);
    
    l_Builder.add (m_StatusMessage,l_Constraints.xy (3,11));
   
    addSubPanel (m_ConnectionPanel);
   
    setIcon  (getIconResource ("icn_database"));
    setLabel (Translatrix.getTranslationString("database:Label"));    
    
    setStationary ("Driver", HSQLDB);
    setStationary ("DSN", HSQLDB.getDSNTemplate());
    setStationary ("Username", new String("none"));
    setStationary ("Password", new String("none"));
    }

//---------------------------------------------------------------------------

private void setDatabaseDriver (DatabaseDriver p_Driver)
    {
    if (p_Driver == null) return;
    
    m_DatabaseDriver = p_Driver;
    //if (m_DSN.getText().length() == 0) m_DSN.setText (p_Driver.getDSNTemplate());
    m_DSN.setText (p_Driver.getDSNTemplate());
    }

//---------------------------------------------------------------------------

//***************************************************************************
//* Class Body                                                              *
//***************************************************************************

//---------------------------------------------------------------------------

public void revertToDefaults ()
    {    
    resetValueChanged ();
   
    setValue ("Driver",  getDefault ("Driver"));
    setValue ("DSN",     getDefault ("DSN"));
    setValue ("Username",getDefault ("Username"));
    setValue ("Password",getDefault ("Password"));
    
    reflectSettings();
    
    super.revertToDefaults ();
    }

//---------------------------------------------------------------------------

public void updateSettings ()
    {
    resetValueChanged ();
    
    setValue ("Driver",  m_DatabaseDriver);
    setValue ("DSN",     m_DSN.getText());
    setValue ("Username",m_Username.getText());
    setValue ("Password",m_Password.getText());
    
    super.updateSettings ();
    }

//---------------------------------------------------------------------------

public void reflectSettings ()
    {
    m_DatabaseDriver = (DatabaseDriver) getValue ("Driver");
    m_DriverSelector.setSelectedItem (m_DatabaseDriver.getDriverName());
   
    m_DSN.setText((String)getValue ("DSN"));
    m_Username.setText((String)getValue ("Username"));
    m_Password.setText((String)getValue ("Password"));    
    
    super.reflectSettings ();
    }

//---------------------------------------------------------------------------

public void addDatabaseDriver (DatabaseDriver p_Driver)
    {
    if (p_Driver == null) return;
    
    m_DatabaseDrivers.add    (p_Driver);
    m_DriverLookup.put       (p_Driver.getDriverName(),new Integer (m_DatabaseDrivers.size()-1));
    m_DriverSelector.addItem (p_Driver.getDriverName());
    
    if (m_DatabaseDrivers.size() >= 2) m_DriverPanel.setVisible (true);
    }
   
//---------------------------------------------------------------------------

public boolean testConnection ()
    {
    Connection  l_Connection = null; 
    
    if (m_DatabaseDriver != null)
        {
        try { 
            Class.forName(m_DatabaseDriver.getDriverClass()).newInstance();
            l_Connection = DriverManager.getConnection(m_DSN.getText(), 
                                                       m_Username.getText(),
                                                       m_Password.getText());
            
            l_Connection.close();
            m_ConnectionStatus = c_Okay;
            m_StatusMessage.setIcon (getIconResource ("icn_okay"));
            m_StatusMessage.setText (Translatrix.getTranslationString("database:StatusOkay"));
            return true;
            }
        catch (SQLException p_Exception) 
            {
            m_Logger.log(Level.WARNING,"Database connection not working: ", p_Exception);
            m_ConnectionStatus = c_Failure;
            m_StatusMessage.setIcon (getIconResource ("icn_failed"));
            m_StatusMessage.setText (Translatrix.getTranslationString("database:StatusFailed"));
            return false;
            }
        catch(Exception p_Exception)
            {
            m_Logger.log(Level.WARNING,"Database connection not working: ", p_Exception);
            m_ConnectionStatus = c_Failure;
            m_StatusMessage.setIcon (getIconResource ("icn_failed"));
            m_StatusMessage.setText (Translatrix.getTranslationString("database:StatusFailed"));
            return false;
            }
        }
    return false;
    }

//---------------------------------------------------------------------------

public void relocalize ()
    {
    super.relocalize();
    
    setSubPanelTitle (m_DriverPanel, Translatrix.getTranslationString("database:DriverTitle"));
    m_DriverDescription.setText (Translatrix.getTranslationString("database:DriverDescription"));
    m_DriverLabel.setText (Translatrix.getTranslationString("database:DriverLabel"));

    setSubPanelTitle (m_ConnectionPanel, Translatrix.getTranslationString("database:DriverTitle"));
    m_ConnectionDescription.setText (Translatrix.getTranslationString("database:Description"));
    m_DSNLabel.setText (Translatrix.getTranslationString("database:DSNLabel"));
    m_UsernameLabel.setText (Translatrix.getTranslationString("database:UsernameLabel"));
    m_PasswordLabel.setText (Translatrix.getTranslationString("database:PasswordLabel"));
    m_TestConnection.setText (Translatrix.getTranslationString("database:TestButton"));
    
    switch (m_ConnectionStatus)
        {
        case c_None     : break;
        
        case c_Okay     : m_StatusMessage.setText (Translatrix.getTranslationString("database:StatusOkay"));
                          break;
            
        case c_Failure  : m_StatusMessage.setText (Translatrix.getTranslationString("database:StatusFailed"));
                          break;
        }
    }

//---------------------------------------------------------------------------

public void actionPerformed(ActionEvent p_ActionEvent) 
    {   
    String  l_DriverName;
    int     l_DriverIndex;
    
    if (p_ActionEvent.getSource () == m_TestConnection) 
        {
        testConnection ();
        }
    
    if (p_ActionEvent.getSource () == m_DriverSelector) 
        {
        l_DriverName = (String) m_DriverSelector.getSelectedItem();
        if (m_DriverLookup.containsKey (l_DriverName))
            {
            l_DriverIndex = ((Integer)m_DriverLookup.get (l_DriverName)).intValue();
            setDatabaseDriver ((DatabaseDriver)m_DatabaseDrivers.elementAt(l_DriverIndex));
            }
        }
    
    if (p_ActionEvent.getSource () == m_DSN)
        {
        if (m_DSN.getText().length()==0)
            {
            if (m_DatabaseDriver != null) m_DSN.setText (m_DatabaseDriver.getDSNTemplate()); 
            }
        }    
    }

//***************************************************************************
//* End Of Class                                                            *
//***************************************************************************
    
}
