/*
 * I18nListPlugin.java
 *
 * Created on July 11, 2005, 3:02 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package lu.tudor.santec.settings;

import java.awt.Component;
import java.beans.DefaultPersistenceDelegate;
import java.util.Locale;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListSelectionModel;
import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.EmptyBorder;

import lu.tudor.santec.i18n.Translatrix;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
*
* @author nmac / jhermen
*/

public class I18nPlugin extends SettingsPlugin {
	private static final long serialVersionUID = 1L;

	private JPanel languagePanel;

	private JList listLanguages;

	private JLabel desc;

	private JLabel label;

	private FormLayout layout;

	private boolean listInsteadComboBox = false;

	private JComboBox cbLanguages;

	public I18nPlugin(String p_Name) {
		super(p_Name);
		buildPanel();
	}

	public I18nPlugin(String p_Name, boolean listInsteadComboBox) {
		super(p_Name);
		this.listInsteadComboBox = listInsteadComboBox;
		buildPanel();

	}

	private void buildPanel() {
		
		setLabel(Translatrix.getTranslationString("i18n:Label"));
		
		if (Translatrix.getLocale() != null ) {
			setStationary("Locale", Translatrix.getLocale());
		} else {
			setStationary("Locale", Locale.getDefault());
		}

		if (this.listInsteadComboBox) {
			this.layout = new FormLayout(
					"right:50dlu,5dlu,fill:60dlu:grow,5dlu",
					"center:pref,5dlu,pref,5dlu,60dlu,5dlu");
		} else {
			this.layout = new FormLayout(
					"right:50dlu,5dlu,fill:70dlu,5dlu:grow",
					"center:pref,5dlu,pref,5dlu");

		}

		CellConstraints cc = new CellConstraints();

		this.languagePanel = createSubPanel(Translatrix
				.getTranslationString("i18n:LanguageTitle"));
		this.languagePanel.setLayout(layout);

		PanelBuilder pBuilder = new PanelBuilder(layout, languagePanel);

		this.desc = new JLabel();
		this.desc.setText(Translatrix.getTranslationString("i18n:Description"));

		pBuilder.add(desc, cc.xyw(1, 1, 3, "fill, center"));

		this.label = new JLabel();
		this.label.setText(Translatrix
				.getTranslationString("i18n:LanguageLabel"));

		pBuilder.add(label, cc.xy(1, 3));

		if (this.listInsteadComboBox) {
			this.listLanguages = new JList(Translatrix.getSupportedLocales());
			this.listLanguages
					.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
			this.listLanguages.setSelectedIndex(0);
			this.listLanguages.setCellRenderer(new LocaleComboBoxRenderer());
			pBuilder.add(new JScrollPane(listLanguages), cc.xywh(3, 3, 1, 3));
		} else {
			this.cbLanguages = new JComboBox(Translatrix.getSupportedLocales());
			this.cbLanguages.setRenderer(new LocaleComboBoxRenderer());
			this.cbLanguages.setSelectedIndex(0);
			pBuilder.add(cbLanguages, cc.xy(3, 3));
		}

		addSubPanel(languagePanel);

		setIcon(getIconResource("icn_i18n"));

		if (this.listInsteadComboBox) {
			this.listLanguages.setSelectedValue((Locale) getDefault("Locale"),
					true);
		} else {
			this.cbLanguages.setSelectedItem((Locale) getDefault("Locale"));
		}
	}

	public void pluginAdded() {
		addPersistenceDelegate(Locale.class, new DefaultPersistenceDelegate(
				new String[] { "language", "country" }));
	}

	public void revertToDefaults() {

		if (this.listInsteadComboBox) {
			this.listLanguages.setSelectedValue((Locale) getDefault("Locale"),
					true);
		} else {
			this.cbLanguages.setSelectedItem((Locale) getDefault("Locale"));
		}

		reflectSettings();
		super.revertToDefaults();
	}

	public void updateSettings() {
		if (this.listInsteadComboBox) {
			setValue("Locale", (Locale) this.listLanguages.getSelectedValue());
		} else {
			Object item = this.cbLanguages.getSelectedItem();
			if (item != null) {
				setValue("Locale", (Locale) item);
			}
		}

		super.updateSettings();
	}

	public void reflectSettings() {
		super.reflectSettings();
		if (this.listInsteadComboBox) {
			this.listLanguages.setSelectedValue((Locale) getValue("Locale"),
					true);
		} else {
			this.cbLanguages.setSelectedItem((Locale) getValue("Locale"));
		}

	}

	public void relocalize() {
		super.relocalize();
		
		setLabel(Translatrix.getTranslationString("i18n:Label"));
		
		setSubPanelTitle(languagePanel, Translatrix
				.getTranslationString("i18n:LanguageTitle"));
		desc.setText(Translatrix.getTranslationString("i18n:Description"));
		label.setText(Translatrix.getTranslationString("i18n:LanguageLabel"));
	}

	public class LocaleComboBoxRenderer extends DefaultListCellRenderer {
		private static final long serialVersionUID = 1L;

		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			} else {
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}
			this.setBorder(new EmptyBorder(1, 5, 1, 1));

			// this.setText(((Locale) value).getDisplayLanguage());
			try {
				this.setText(Translatrix.getTranslationString("i18n:language_"
						+ ((Locale) value).getLanguage().toUpperCase()));
				this.setIcon(getFlagIcon((Locale) value));
			} catch (Exception e) {
			}
			return this;
		}
	}

	/**
	 * returns the matching flag-icon for the Locales Language
	 * 
	 * @param l
	 *            the Locale
	 * @return the Flag-Icon
	 */
	public static ImageIcon getFlagIcon(Locale l) {
		java.net.URL imgURL = I18nPlugin.class
				.getResource("resources/icn_flag_"
						+ l.getLanguage().toUpperCase() + ".gif");
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		}
		return new ImageIcon(I18nPlugin.class
				.getResource("resources/icn_flag_" + "default" + ".gif"));
	}

}
