package lu.tudor.santec.settings;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.util.logging.ConsoleHandler;
import java.util.logging.FileHandler;
import java.util.logging.Formatter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import java.util.logging.SocketHandler;
import java.util.logging.XMLFormatter;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;

import lu.tudor.santec.i18n.Translatrix;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * 
 * @author jhermen
 */

// ***************************************************************************
// * Class Definition and Members *
// ***************************************************************************
public class LoggingPlugin extends SettingsPlugin implements SettingListener, ActionListener {

	private static final long serialVersionUID = 1L;

	private JCheckBox fileEnabled, fileAppend, consoleEnabled, socketEnabled, showConsoleWindow;

	private JSpinner fileNumber, fileSize;

	private JTextField fileName, socketHost;
	
	private JSpinner socketPort;

	private JButton chooseFile;

	private JComboBox logLevel, fileType, socketType;

	private JLabel fileNameLabel, logLevelLabel, fileTypeLabel, fileSizeLabel,
			fileNumberLabel, socketHostLabel, socketTypeLabel, socketPortLabel;

	private JFileChooser fileChooser;

	private JPanel filePanel, consolePanel, socketPanel, generalPanel, windowPanel;

	public final static String NAME = "logging";
	
	private static Level[] LOGLEVELS = { Level.ALL, Level.FINEST, Level.FINER,
			Level.FINE, Level.CONFIG, Level.INFO, Level.WARNING, Level.SEVERE };

	private static Logger logger = Logger
			.getLogger("lu.tudor.santec.settings.LoggingPlugin");
	
	private Console console;

	private Logger generalLogger = Logger.getLogger("");


	// ***************************************************************************
	// * Constructor *
	// ***************************************************************************

	public LoggingPlugin() {
        super (NAME);
		setIcon(getIconResource("icn_logging"));
		this.setStationaryValues();
		this.buildPanel();
		relocalize();
	}
	
	/**
	 * Creates a new instance of LoggingPlugin
	 */
	public LoggingPlugin(String p_Name) {
                super (p_Name);
		setIcon(getIconResource("icn_logging"));
		this.setStationaryValues();
		this.buildPanel();
		relocalize();
	}

	// ***************************************************************************
	// * Class Primitives *
	// ***************************************************************************

	/**
	 * adds the components to the panel
	 */
	private void buildPanel() {

		initComponents();
		CellConstraints cc = new CellConstraints();

		// build the Layout for general
		generalPanel = createSubPanel(Translatrix
				.getTranslationString("logging:GeneralSettings"));
		FormLayout generalLayout = new FormLayout(
				"40dlu, 2dlu, 40dlu, 2dlu, 40dlu, 2dlu, 40dlu, 2dlu, pref:grow", "2dlu, pref, 2dlu");
		generalPanel.setLayout(generalLayout);
		generalPanel.add(this.logLevelLabel, cc.xy(1, 2));
		generalPanel.add(this.logLevel, cc.xyw(3, 2,3));
		addSubPanel(generalPanel);

		// build the Layout for console
		consolePanel = createSubPanel(Translatrix
				.getTranslationString("logging:LogToConsole"));
		FormLayout consoleLayout = new FormLayout(
				"40dlu, 2dlu, 40dlu, 2dlu, 40dlu, 2dlu, 25dlu, 2dlu, pref:grow", "pref, 2dlu"); //$NON-NLS-1$
		consolePanel.setLayout(consoleLayout);
		// add Fields for console
		consolePanel.add(this.consoleEnabled, cc.xyw(1, 1, 3));
		addSubPanel(consolePanel);
		
		// build the Layout for window
		windowPanel = createSubPanel(Translatrix
				.getTranslationString("logging:LogToWindow"));
		FormLayout windowLayout = new FormLayout(
				"40dlu, 2dlu, 40dlu, 2dlu, 40dlu, 2dlu, 25dlu, 2dlu, pref:grow", "pref, 2dlu"); //$NON-NLS-1$
		windowPanel.setLayout(windowLayout);
		// add Fields for console
		windowPanel.add(this.showConsoleWindow, cc.xyw(1, 1, 5));
		addSubPanel(windowPanel);

		// build the Layout for file
		filePanel = createSubPanel(Translatrix
				.getTranslationString("logging:LogToFile"));
		FormLayout fileLayout = new FormLayout(
				"40dlu, 2dlu, 40dlu, 2dlu, 40dlu, 2dlu, 25dlu, 2dlu, pref:grow", "pref, 2dlu, pref, 2dlu, pref, 2dlu, pref, 2dlu"); //$NON-NLS-1$
		filePanel.setLayout(fileLayout);
		JPanel fileChooserPanel = new JPanel(new BorderLayout());
		fileChooserPanel.add(this.fileName, BorderLayout.CENTER);
		fileChooserPanel.add(this.chooseFile, BorderLayout.EAST);
		// add Fields for file
		filePanel.add(this.fileEnabled, cc.xyw(1, 1, 3));
		filePanel.add(this.fileNameLabel, cc.xy(1, 3));
		filePanel.add(fileChooserPanel, cc.xyw(3, 3, 7));
		filePanel.add(this.fileTypeLabel, cc.xy(1, 5));
		filePanel.add(this.fileType, cc.xyw(3, 5, 3));
		filePanel.add(this.fileAppend, cc.xyw(6, 5, 4));
		filePanel.add(this.fileNumberLabel, cc.xy(1, 7));
		filePanel.add(this.fileNumber, cc.xy(3, 7));
		filePanel.add(this.fileSizeLabel, cc.xy(5, 7));
		filePanel.add(this.fileSize, cc.xy(7, 7));
		addSubPanel(filePanel);

		// build the Layout for logging host
		socketPanel = createSubPanel(Translatrix
				.getTranslationString("logging:LogToLoggingHost"));
		FormLayout socketLayout = new FormLayout(
				"40dlu, 2dlu, 40dlu, 2dlu, 40dlu, 2dlu, 25dlu, 2dlu, pref:grow", "pref, 2dlu, pref, 2dlu,pref, 2dlu"); //$NON-NLS-1$
		socketPanel.setLayout(socketLayout);
		// add Fields for logging host
		socketPanel.add(this.socketEnabled, cc.xyw(1, 1, 3));
		socketPanel.add(this.socketHostLabel, cc.xy(1, 3));
		socketPanel.add(this.socketHost, cc.xyw(3, 3, 3));
		socketPanel.add(this.socketPortLabel, cc.xy(7, 3));
		socketPanel.add(this.socketPort, cc.xy(9, 3));
		socketPanel.add(this.socketTypeLabel, cc.xy(1, 5));
		socketPanel.add(this.socketType, cc.xyw(3, 5, 3));
		addSubPanel(socketPanel);
	}

	/**
	 * initialises the Components
	 */
	private void initComponents() {

		// elements for general
		this.logLevelLabel = new JLabel();
		this.logLevel = new JComboBox(LOGLEVELS);
		
		// elements for console
		this.consoleEnabled = new JCheckBox();
		
		// elements for window
		this.showConsoleWindow = new JCheckBox();
		this.showConsoleWindow.addActionListener(this);

		// elements for file
		this.fileEnabled = new JCheckBox();
		this.chooseFile = new JButton(new AbstractAction("...") {
			private static final long serialVersionUID = 1L;

			
			public void actionPerformed(ActionEvent e) {
				fileChooser = new JFileChooser();
				fileChooser.showOpenDialog(getParent());
				if (fileChooser.getSelectedFile() != null) {
					fileName.setText(fileChooser.getSelectedFile()
							.getAbsolutePath());
				}
			}
		});
		this.fileNameLabel = new JLabel();
		this.fileName = new JTextField();
		this.fileTypeLabel = new JLabel();
		Formatter[] formatters = { new SimpleFormatter(), new XMLFormatter() };
		this.fileType = new JComboBox(formatters);
		this.fileType.setRenderer(new FormatterComboBoxRenderer());
		this.fileAppend = new JCheckBox();
		this.fileNumberLabel = new JLabel();
		this.fileNumber = new JSpinner(new SpinnerNumberModel(1, 1, 20, 1));
		this.fileSizeLabel = new JLabel();
		this.fileSize = new JSpinner(new SpinnerNumberModel(1, 1, 20, 1));

		// elements for socket
		this.socketEnabled = new JCheckBox();
		this.socketTypeLabel = new JLabel();
		this.socketType = new JComboBox(formatters);
		this.socketType.setRenderer(new FormatterComboBoxRenderer());
		this.socketHostLabel = new JLabel();
		this.socketHost = new JTextField();
		this.socketPortLabel = new JLabel();
		this.socketPort = new JSpinner(new SpinnerNumberModel(5104, 1024, 65535, 1));
		
	}

	// ***************************************************************************
	// * Class Body *
	// ***************************************************************************

	public void setStationaryValues() {
		setStationary("LogLevel", new Integer(6));

		// console logging
		setStationary("ConsoleEnabled",	new Boolean(true));

		// file logging
		setStationary("FileEnabled", new Boolean(false));
		setStationary("FileName", "");
		setStationary("FileType", new Integer(0));
		setStationary("FileNumber", new Integer(1));
		setStationary("FileSize", new Integer(1));
		setStationary("FileAppend", new Boolean(false));

		// socket logging
		setStationary("SocketEnabled", new Boolean(false));
		setStationary("SocketType", new Integer(0));
		setStationary("SocketHost", "localhost");
		setStationary("SocketPort", new Integer(1234));
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see lu.tudor.santec.settings.SettingsPlugin#revertToDefaults()
	 */
	public void revertToDefaults() {

		this.logLevel.setSelectedIndex(((Integer) getDefault("LogLevel"))
				.intValue());

		// console logging
		this.consoleEnabled
				.setSelected(((Boolean) getDefault("ConsoleEnabled"))
						.booleanValue());

		// file logging
		this.fileEnabled.setSelected(((Boolean) getDefault("FileEnabled"))
				.booleanValue());
		this.fileName.setText((String) getDefault("FileName"));
		this.fileType.setSelectedIndex(((Integer) getDefault("FileType"))
				.intValue());
		this.fileNumber.setValue(getDefault("FileNumber"));
		this.fileSize.setValue(getDefault("FileSize"));
		this.fileAppend.setSelected(((Boolean) getDefault("FileAppend"))
				.booleanValue());

		// socket logging
		this.socketEnabled.setSelected(((Boolean) getDefault("SocketEnabled"))
				.booleanValue());
		this.socketType.setSelectedIndex(((Integer) getDefault("SocketType"))
				.intValue());
		this.socketHost.setText((String) getDefault("SocketHost"));
		this.socketPort.setValue((Integer) getDefault("SocketPort"));

		reflectSettings();
		super.revertToDefaults();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see lu.tudor.santec.settings.SettingsPlugin#updateSettings()
	 */
	public void updateSettings() {

		setValue("LogLevel", new Integer(this.logLevel.getSelectedIndex()));

		// console logging
		setValue("ConsoleEnabled",
				new Boolean(this.consoleEnabled.isSelected()));

		// file logging
		setValue("FileEnabled", new Boolean(this.fileEnabled.isSelected()));
		setValue("FileName", new String(this.fileName.getText()));
		setValue("FileType", new Integer(this.fileType.getSelectedIndex()));
		setValue("FileNumber", (Integer) (this.fileNumber.getValue()));
		setValue("FileSize", (Integer) (this.fileSize.getValue()));
		setValue("FileAppend", new Boolean(this.fileAppend.isSelected()));

		// socket logging
		setValue("SocketEnabled", new Boolean(this.socketEnabled.isSelected()));
		setValue("SocketType", new Integer(this.socketType.getSelectedIndex()));
		setValue("SocketHost", new String(this.socketHost.getText()));
		setValue("SocketPort", (Integer) this.socketPort.getValue());

		super.updateSettings();
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see lu.tudor.santec.settings.SettingsPlugin#reflectSettings()
	 */
	public void reflectSettings() {
		super.reflectSettings();
		try {

			this.logLevel.setSelectedIndex(((Integer) getValue("LogLevel"))
					.intValue());

			// console logging
			this.consoleEnabled
					.setSelected(((Boolean) getValue("ConsoleEnabled"))
							.booleanValue());

			// file logging
			this.fileEnabled.setSelected(((Boolean) getValue("FileEnabled"))
					.booleanValue());
			this.fileName.setText((String) getValue("FileName"));
			this.fileType.setSelectedIndex(((Integer) getValue("FileType"))
					.intValue());
			this.fileNumber.setValue(getValue("FileNumber"));
			this.fileSize.setValue(getValue("FileSize"));
			this.fileAppend.setSelected(((Boolean) getValue("FileAppend"))
					.booleanValue());

			// socket logging
			this.socketEnabled
					.setSelected(((Boolean) getValue("SocketEnabled"))
							.booleanValue());
			this.socketType.setSelectedIndex(((Integer) getValue("SocketType"))
					.intValue());
			this.socketHost.setText((String) getValue("SocketHost"));
			this.socketPort.setValue((Integer) getValue("SocketPort"));

			this.updateLoggers();

		} catch (Exception e) {
			System.err.println("Could not load settings");
			e.printStackTrace();
		}
	}

	/**
	 * delete old loggers and create the new specified ones.
	 */
	public void updateLoggers() {
		// remove all existing handlers
		Handler[] handlers = generalLogger.getHandlers();
		for (int i = 0; i < handlers.length; i++) {
			generalLogger.removeHandler(handlers[i]);
		}
		// set Loglevel
		generalLogger.setLevel((Level) this.logLevel.getSelectedItem());
		logger.info("LogLevel set to: " + this.logLevel.getSelectedItem());

		// add console handler
		try {
			if (this.consoleEnabled.isSelected()) {
				Handler ch = new ConsoleHandler();
				generalLogger.addHandler(ch);
				logger.info("Added logging to console");
			}
		} catch (Exception e) {
			System.err.println("Could not create console logger: "
					+ e.getLocalizedMessage());
			logger.warning("Could not create console logger: "
					+ e.getLocalizedMessage());
		}

		// add file handler
		try {
			if (this.fileEnabled.isSelected()) {
				Handler fh = new FileHandler(
						this.fileName.getText(),
						((Integer) this.fileSize.getValue()).intValue() * 1024 * 1024,
						((Integer) this.fileNumber.getValue()).intValue(),
						this.fileAppend.isSelected());
				fh.setFormatter((Formatter) this.fileType.getSelectedItem());
				generalLogger.addHandler(fh);
				logger
						.info("Added logging to file: "
								+ this.fileName.getText());
			}
		} catch (Exception e) {
			System.err.println("Could not create file logger: "
					+ e.getLocalizedMessage());
			logger.warning("Could not create file logger: "
					+ e.getLocalizedMessage());
		}

		// add socket handler
		try {
			if (this.socketEnabled.isSelected()) {
				Handler sh = new SocketHandler(this.socketHost.getText(), ((Integer) this.socketPort.getValue()).intValue());
				generalLogger.addHandler(sh);
				sh.setFormatter((Formatter) this.fileType.getSelectedItem());
				logger.info("Added logging to socket: "
						+ this.socketHost.getText() + ":"
						+ this.socketPort.getValue());
			}
		} catch (Exception e) {
			System.err.println("Could not create socket logger: "
					+ e.getLocalizedMessage());
			logger.warning("Could not create socket logger: "
					+ e.getLocalizedMessage());
		}

	}

	/**
	 * Method is part of the Relocalizable interface. The method does everything
	 * required to reflect changes of active Locale
	 */
	public void relocalize() {
		
		setLabel(Translatrix.getTranslationString("logging:Logging"));
		
		setSubPanelTitle(generalPanel,Translatrix	.getTranslationString("logging:GeneralSettings"));
		this.logLevelLabel.setText(Translatrix.getTranslationString("logging:LogLevel"));
		this.consoleEnabled.setText(Translatrix.getTranslationString("logging:Enable"));
		
		setSubPanelTitle(consolePanel,Translatrix	.getTranslationString("logging:LogToConsole"));
		this.consoleEnabled.setText(Translatrix.getTranslationString("logging:Enable"));
		
		setSubPanelTitle(windowPanel,Translatrix	.getTranslationString("logging:LogToWindow"));
		this.showConsoleWindow.setText(Translatrix.getTranslationString("logging:Enable"));
		
		setSubPanelTitle(filePanel,Translatrix.getTranslationString("logging:LogToFile"));
		this.fileEnabled.setText(Translatrix.getTranslationString("logging:Enable"));
		this.fileNameLabel.setText(Translatrix.getTranslationString("logging:LogFile"));
		this.fileTypeLabel.setText(Translatrix.getTranslationString("logging:Type"));
		this.fileAppend.setText(Translatrix.getTranslationString("logging:Append"));
		this.fileNumberLabel.setText(Translatrix.getTranslationString("logging:FileNr"));
		this.fileSizeLabel.setText(Translatrix.getTranslationString("logging:FileSize"));
		
		setSubPanelTitle(socketPanel,Translatrix.getTranslationString("logging:LogToLoggingHost"));
		this.socketEnabled.setText(Translatrix.getTranslationString("logging:Enable"));
		this.socketTypeLabel.setText(Translatrix.getTranslationString("logging:Type"));
		this.socketHostLabel.setText(Translatrix.getTranslationString("logging:Host"));
		this.socketPortLabel.setText(Translatrix.getTranslationString("logging:Port"));
		
	}

	// ***************************************************************************
	// * End Of Class *
	// ***************************************************************************

	public class FormatterComboBoxRenderer extends JLabel implements
			ListCellRenderer {

		private static final long serialVersionUID = 1L;

		/**
		 * builds a new DefaultComboboxRenderer
		 */
		public FormatterComboBoxRenderer() {
			setOpaque(true);
			setHorizontalAlignment(LEFT);
			setVerticalAlignment(CENTER);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList,
		 *      java.lang.Object, int, boolean, boolean)
		 */
		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			} else {
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}
			try {
				this.setText(" "+value.getClass().getName().substring(
						value.getClass().getName().lastIndexOf(".") + 1));
			} catch (Exception e) {
				this.setText("");
			}
			return this;
		}

	}

	public void settingChanged(SettingEvent p_Event) {
		if (p_Event.getPluginName().equals(this.getName())) {
			this.updateLoggers();
		}
	}
	
	/* (non-Javadoc)
	 * @see lu.tudor.santec.settings.SettingsPlugin#pluginAdded()
	 */
	public void pluginAdded() {
		m_Owner.addSettingListener(this);
	}
	    
    public class Console extends JFrame {
		private static final long serialVersionUID = 1L;
		PipedInputStream piOut;
        PipedInputStream piErr;
        PipedOutputStream poOut;
        PipedOutputStream poErr;
        JTextArea textArea = new JTextArea();
        PrintStream oldOut;
        PrintStream oldErr;
		private ConsoleHandler console2window;
        
        public Console() throws IOException {
            super("Logging Window");
        	this.setIconImage(getIconResource("icn_logging").getImage());
        	
            this.oldOut = System.out;
            this.oldErr = System.err;
        	
        	// Set up System.out
            piOut = new PipedInputStream();
            poOut = new PipedOutputStream(piOut);            
            System.setOut(new PrintStream(poOut, true));
            
            // Set up System.err
            piErr = new PipedInputStream();
            poErr = new PipedOutputStream(piErr);
            System.setErr(new PrintStream(poErr, true));
            
            console2window = new ConsoleHandler();

            generalLogger.addHandler(console2window);
			
            // Add a scrolling text area
            textArea.setEditable(false);
            textArea.setRows(20);
            textArea.setColumns(50);
            getContentPane().add(new JScrollPane(textArea), BorderLayout.CENTER);
            pack();
            setVisible(true);
    
            // Create reader threads
            new ReaderThread(piOut, "OUT").start();
            new ReaderThread(piErr, "ERR").start();      
                        
			System.out.println("Testing System.out");
			System.err.println("Testing System.err");
			logger.warning("Testing logging to Logwindow");
        }
    
        class ReaderThread extends Thread {
            PipedInputStream pi;
    
            ReaderThread(PipedInputStream pi, String prefix) {
                this.pi = pi;
            }
    
            public void run() {
                final byte[] buf = new byte[1024];
                try {
                    while (true) {
                        final int len = pi.read(buf);
                        if (len == -1) {
                        	break;
                        }
                        SwingUtilities.invokeLater(new Runnable() {
                            public void run() {
                            	textArea.append(new String(buf, 0, len));
                                // Make sure the last line is always visible
                                textArea.setCaretPosition(textArea.getDocument().getLength());
    
                                // Keep the text area down to a certain character size
                                int idealSize = 1000;
                                int maxExcess = 500;
                                int excess = textArea.getDocument().getLength() - idealSize;
                                if (excess >= maxExcess) {
                                    textArea.replaceRange("", 0, excess);
                                }
                                
                            }
                        });
                    }
                } catch (IOException e) {
                	e.printStackTrace();
                }
            }
        }
        
        public void stop() {
        	System.setOut(this.oldOut);
        	System.setErr(this.oldErr);
        	Logger.getLogger("").removeHandler(console2window);
			this.dispose();
        }       
    }

	public void actionPerformed(ActionEvent arg0) {
		if (arg0.getSource().equals(this.showConsoleWindow)) {
			if (this.showConsoleWindow.isSelected()) {
				try {
					console = new Console();
				} catch (IOException e) {
					e.printStackTrace();
				}
			} else if (console != null) {
				console.stop();
			}
		}
	}
	
}
