 /*
 * I18nListPlugin.java
 *
 * Created on July 11, 2005, 3:02 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package lu.tudor.santec.settings;

import java.awt.Component;
import java.util.logging.Logger;

import javax.swing.DefaultListCellRenderer;
import javax.swing.DefaultListSelectionModel;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

import lu.tudor.santec.i18n.Translatrix;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * this class offers the possibility to set the look and feel of the Application
 * @author jhermen
 */

public class LookAndFeelPlugin extends SettingsPlugin implements SettingListener {
	private static final long serialVersionUID = 1L;

	private JPanel lnfPanel;

	private JList listLookAndFeel;

	private JLabel desc;

	private JLabel label;

	private FormLayout layout;

	String[] lookAndFeels;
	
	private Object[] windows;
	
	private static Logger logger = Logger
	.getLogger("lu.tudor.santec.settings.LookAndFeelPlugin");

	/**
	 * creates a new LookAndFeelPlugin Settings Plugin
	 * @param p_Name
	 *            the name of the panel
	 */
	public LookAndFeelPlugin(String p_Name) {
		super(p_Name);
		UIManager.LookAndFeelInfo[] arr = UIManager.getInstalledLookAndFeels();
		lookAndFeels = new String[arr.length];
		for (int i = 0; i < arr.length; i++) {
			lookAndFeels[i] = arr[i].getClassName();
		}		
		buildPanel();
		relocalize();
	}
	
	/**
	 * creates a new LookAndFeelPlugin Settings Plugin
	 * @param p_Name
	 *            the name of the panel
	 * @param windows reference to all windows to set the look and feel
	 */
	public LookAndFeelPlugin(String p_Name, Object[] windows) {
		super(p_Name);
		UIManager.LookAndFeelInfo[] arr = UIManager.getInstalledLookAndFeels();
		lookAndFeels = new String[arr.length];
		for (int i = 0; i < arr.length; i++) {
			lookAndFeels[i] = arr[i].getClassName();
		}
		this.windows = windows;
		buildPanel();
		relocalize();
	}
	
	/**
	 * builds the panel
	 */
	private void buildPanel() {
		setLabel(Translatrix.getTranslationString("lnf:Label"));
		UIManager.LookAndFeelInfo lnfi = new UIManager.LookAndFeelInfo(
				UIManager.getLookAndFeel().getName(), UIManager
						.getLookAndFeel().getClass().getName());
		
		setStationary("LookAndFeel", lnfi);

		this.layout = new FormLayout("right:50dlu,5dlu,fill:60dlu:grow,5dlu",
				"center:pref,5dlu,pref,5dlu,60dlu,5dlu");
		CellConstraints cc = new CellConstraints();

		this.lnfPanel = createSubPanel(Translatrix
				.getTranslationString("lnf:Title"));
		this.lnfPanel.setLayout(layout);

		PanelBuilder pBuilder = new PanelBuilder(layout, lnfPanel);

		this.desc = new JLabel("desc");

		pBuilder.add(desc, cc.xyw(1, 1, 3, "fill, center"));

		this.label = new JLabel("label");

		pBuilder.add(label, cc.xy(1, 3));

		this.listLookAndFeel = new JList(this.lookAndFeels);
		this.listLookAndFeel
				.setSelectionMode(DefaultListSelectionModel.SINGLE_SELECTION);
		this.listLookAndFeel.setSelectedIndex(0);
		this.listLookAndFeel.setCellRenderer(new LnFComboBoxRenderer());
		pBuilder.add(new JScrollPane(listLookAndFeel), cc.xywh(3, 3, 1, 3));

		addSubPanel(lnfPanel);

		setIcon(getIconResource("icn_looknfeel"));

		this.listLookAndFeel.setSelectedValue(getDefault("LookAndFeel"), true);

		setLabel(Translatrix.getTranslationString("lnf:Label"));
	}

	/* (non-Javadoc)
	 * @see lu.tudor.santec.settings.SettingsPlugin#pluginAdded()
	 */
	public void pluginAdded() {
		m_Owner.addSettingListener(this);
//		addPersistenceDelegate(UIManager.LookAndFeelInfo.class,
//				new DefaultPersistenceDelegate(new String[] { "name",
//						"className" }));
	}

	/* (non-Javadoc)
	 * @see lu.tudor.santec.settings.SettingsPlugin#revertToDefaults()
	 */
	public void revertToDefaults() {

		setSelectedItem(getDefault("LookAndFeel").toString());

		reflectSettings();
		super.revertToDefaults();
	}

	/* (non-Javadoc)
	 * @see lu.tudor.santec.settings.SettingsPlugin#updateSettings()
	 */
	public void updateSettings() {
		setValue("LookAndFeel",this.listLookAndFeel.getSelectedValue());
		super.updateSettings();
	}

	/* (non-Javadoc)
	 * @see lu.tudor.santec.settings.SettingsPlugin#reflectSettings()
	 */
	public void reflectSettings() {
		super.reflectSettings();
		setSelectedItem(getValue("LookAndFeel").toString());
	}

	/* (non-Javadoc)
	 * @see lu.tudor.santec.i18n.Relocalizable#relocalize()
	 */
	public void relocalize() {
		setLabel(Translatrix.getTranslationString("lnf:Label"));
		setSubPanelTitle(lnfPanel, Translatrix
				.getTranslationString("lnf:Title"));
		this.desc.setText(Translatrix.getTranslationString("lnf:Description"));
		this.label.setText(Translatrix.getTranslationString("lnf:LnFLabel"));
	}

	/**
	 * set the given look and feel for the current program
	 * @param lookandFeel
	 */
	public void setLookAndFeel(String lookandFeel) {
			try {
				UIManager.setLookAndFeel(lookandFeel);
			} catch (Exception e) {
				e.printStackTrace();
			}
			for (int i = 0; i < windows.length; i++) {
				if (windows[i] instanceof JFrame) {
					SwingUtilities.updateComponentTreeUI((JFrame)windows[i]);
					((JFrame)windows[i]).pack();	
				} else if (windows[i] instanceof JDialog) {
					SwingUtilities.updateComponentTreeUI((JDialog)windows[i]);
					((JDialog)windows[i]).pack();	
				}
			}
			SwingUtilities.updateComponentTreeUI((JDialog)m_Owner);
			((JDialog)m_Owner).pack();	
	
			logger.info("set LookAndFeel to: "+lookandFeel );
			
	}

	/**
	 * set the given LookAndFeelInfo to the comboBox
	 * @param lnfi
	 */
	private void setSelectedItem(String lnfi) {
		for (int i = 0; i < this.lookAndFeels.length; i++) {
			if (this.lookAndFeels[i].equals(lnfi)) {
				this.listLookAndFeel.setSelectedIndex(i);
				return;
			}
		}

	}

	/**
	 * Renderer for the LookAndFeelInfo ComboBox
	 * @author hermenj
	 *
	 */
	public class LnFComboBoxRenderer extends DefaultListCellRenderer {
		private static final long serialVersionUID = 1L;

		public Component getListCellRendererComponent(JList list, Object value,
				int index, boolean isSelected, boolean cellHasFocus) {
			if (isSelected) {
				setBackground(list.getSelectionBackground());
				setForeground(list.getSelectionForeground());
			} else {
				setBackground(list.getBackground());
				setForeground(list.getForeground());
			}
			this.setBorder(new EmptyBorder(1, 5, 1, 1));

			try {
				this.setText(value.toString());
			} catch (Exception e) {
			}
			return this;
		}
	}

	/* (non-Javadoc)
	 * @see lu.tudor.santec.settings.SettingListener#settingChanged(lu.tudor.santec.settings.SettingEvent)
	 */
	public void settingChanged(SettingEvent p_Event) {
		if (p_Event.getPluginName().equals(this.getName())) {
			if (windows != null ) {
				this.setLookAndFeel(this.listLookAndFeel.getSelectedValue().toString());
			}
			logger.info("setting of LookAndFeel changed to: "+this.listLookAndFeel.getSelectedValue() );
		}
	}

}
