/*
 * SettingsPlugin.java
 *
 * Created on July 4, 2005, 2:00 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */
package lu.tudor.santec.settings;

import java.io.File;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.IOException;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.Font;
import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;

import java.util.Vector;
import java.util.Hashtable;
import java.util.logging.Logger;
import java.util.logging.Level;

import java.beans.XMLEncoder;
import java.beans.XMLDecoder;
import java.beans.PersistenceDelegate;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JToggleButton;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.BorderFactory;

import javax.swing.border.TitledBorder;
import javax.swing.border.EmptyBorder;

import java.net.URL;

import lu.tudor.santec.widgets.LayoutToolkit;

import lu.tudor.santec.i18n.Translatrix;
import lu.tudor.santec.i18n.Relocalizable;

/**
 *
 * @author nmac
 */

//***************************************************************************
//* Class Definition and Members                                            *
//***************************************************************************

public class SettingsPlugin extends JPanel implements Relocalizable 
    {
    private static Logger m_Logger = Logger.getLogger ("lu.tudor.santec.settings.SettingsPlugin");

    protected   SettingsPanel m_Owner;
//    private   int           m_PluginID;
    private   String        m_Name;
    private   Vector        m_Extensions;
    private   Vector        m_SubPanels;
    private   Hashtable     m_Defaults;
    private   Hashtable     m_Stationaries;
    
    private   JLabel        m_Title;      
    private   JToggleButton m_Help;
    
    private   ImageIcon     m_Icon;
    private   String        m_Label;
    private   String        m_Tooltip;
    private   JScrollPane   m_Scroller;
    private   JPanel        m_Content;
    private   JScrollPane   m_ContentScroller;
    private   JPanel        m_ContentFiller;
    private   boolean       m_HasDefaults;
    private   boolean       m_ValuesChanged;
    
    private static final Color c_TitleBackground = new Color (193,210,238);
    private static final EmptyBorder c_Padding   = new EmptyBorder (5,5,5,5);
    private static final int   c_Inset           = 5;
    
//***************************************************************************
//* Constructor                                                             *
//***************************************************************************
    
/** Creates a new instance of SettingsPlugin */

public SettingsPlugin (String p_Name) 
    {
    GridBagLayout       l_Layout;
    GridBagLayout       l_ContentLayout;
    GridBagConstraints  l_Constraints;
    Font                l_Font;
    Font                l_TitleFont;
    ImageIcon           l_HelpIcon;
    
    m_HasDefaults       = false;
    m_ValuesChanged     = false;

    m_Name              = p_Name;
    
    m_SubPanels         = new Vector ();
    m_Extensions        = new Vector ();
    
    m_Defaults          = new Hashtable ();
    m_Stationaries      = new Hashtable ();
    
    l_Layout            = new GridBagLayout ();
    l_Constraints       = new GridBagConstraints ();
    
    this.setLayout (l_Layout);
    
    l_Constraints = LayoutToolkit.buildConstraint (0,0,1,1,99,1,5,5);
    l_Constraints.fill		= GridBagConstraints.HORIZONTAL;
    l_Constraints.anchor	= GridBagConstraints.NORTH;

    m_Title = new JLabel ();
    m_Title.setOpaque (true);
    m_Title.setBackground (c_TitleBackground);
    m_Title.setBorder (c_Padding);
    m_Title.setHorizontalAlignment(JLabel.CENTER);
    m_Title.setVerticalAlignment(JLabel.CENTER);

    l_Font = m_Title.getFont ();
    l_TitleFont = l_Font.deriveFont (Font.BOLD,l_Font.getSize() + 2);
    m_Title.setFont (l_TitleFont);
    
    l_Layout.setConstraints (m_Title,l_Constraints);
    this.add (m_Title);
    
    l_Constraints = LayoutToolkit.buildConstraint (1,0,1,1,1,1,0,0);
    l_Constraints.fill		= GridBagConstraints.NONE;
    l_Constraints.anchor	= GridBagConstraints.CENTER;
    
    l_HelpIcon = getIconResource("icn_help");
    m_Help     = new JToggleButton (l_HelpIcon);
    m_Help.setEnabled (false);
    l_Layout.setConstraints (m_Help,l_Constraints);
    this.add (m_Help);
    
    l_Constraints = LayoutToolkit.buildConstraint (0,1,2,1,100,99,5,5);
    l_Constraints.fill		= GridBagConstraints.BOTH;
    l_Constraints.anchor	= GridBagConstraints.NORTH;
    
    m_Content = new JPanel ();
    l_ContentLayout = new GridBagLayout ();
    m_Content.setLayout (l_ContentLayout);
      
    m_ContentScroller = new JScrollPane (m_Content);
    m_ContentScroller.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    m_ContentScroller.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
 
    l_Layout.setConstraints (m_ContentScroller,l_Constraints);
    this.add (m_ContentScroller);
    
    m_ContentFiller = new JPanel ();
    
    l_Constraints = LayoutToolkit.buildConstraint (0,GridBagConstraints.RELATIVE,1,1,100,99,0,0);
    l_Constraints.fill		= GridBagConstraints.BOTH;
    l_Constraints.anchor	= GridBagConstraints.NORTH;
    
    l_ContentLayout.setConstraints (m_ContentFiller,l_Constraints);
    m_Content.add (m_ContentFiller);
       
    m_Content.setPreferredSize (getPreferredSize());        
        
    m_Icon          = getIconResource ("icn_Plugin");
    m_Label         = Translatrix.getTranslationString ("PluginLabel");
    m_Tooltip       = Translatrix.getTranslationString ("PluginTooltip");    
    }
    
//***************************************************************************
//* Class Primitives                                                        *
//***************************************************************************

//---------------------------------------------------------------------------
/** returns the ImageIcon Resource specified by p_IconName.
 * @param p_IconName specifies the name of the icon to get from application resources.
 * @return the ImageIcon specified by p_IconName, or <CODE>null</CODE> if specified does not exist.
 */
//---------------------------------------------------------------------------

protected static ImageIcon getIconResource (String p_IconName) 
    {
    String      l_IconPath;   
    URL         l_Location;   
    ImageIcon   l_Icon;   
    
    l_IconPath = new String ("resources/" + p_IconName + ".gif");
    l_Location = SettingsPlugin.class.getResource (l_IconPath);
    
    if (l_Location != null) l_Icon = new ImageIcon (l_Location);
                       else l_Icon = new ImageIcon (l_IconPath);
    
    if (l_Icon != null) return l_Icon;
                   else return new ImageIcon (""); 
    }

//---------------------------------------------------------------------------
/**
 * loads settings from settings file. Default settings file is <I>settings.xml</I>.
 * Use setSettingsFile method to change default settings file.
 * @see setSettingsFile
 */
//---------------------------------------------------------------------------

public boolean loadDefaults (File p_DefaultsFile)
    {
    FileInputStream     l_Stream         = null;
    BufferedInputStream l_BufferedStream = null;
    XMLDecoder          l_Decoder        = null;
    boolean             l_Success        = false;
    
    if ((p_DefaultsFile != null) && p_DefaultsFile.isFile())
        {
        try {
            l_Stream         = new FileInputStream (p_DefaultsFile);
            l_BufferedStream = new BufferedInputStream (l_Stream);
              
            //=======================================================================
            // De-Serialize Defaults from XML
            //=======================================================================

            l_Decoder = new XMLDecoder (l_BufferedStream);

            try { 
                m_Defaults = (Hashtable) l_Decoder.readObject();
                } 
            finally // No matter what happens, close object stream.
                {                   
                l_Decoder.close();
                }

            l_BufferedStream.close();
            l_Success       = true;
            } 
        catch(Exception p_Exception)         
            {
            m_Logger.log (Level.WARNING,"Failed to load Defaults file " + p_DefaultsFile.toString()+ "!",p_Exception);
            }     
        }
        
    m_HasDefaults = l_Success;
    
    return l_Success;
    }
           
//***************************************************************************
//* Class Body                                                              *
//***************************************************************************

//===========================================================================
//= Plugin Helper methods                                                   =
//===========================================================================

//---------------------------------------------------------------------------
/**
 * the createSubPanel creates a new JPanel with a titled border.
 * @param p_PanelTitle specifies the title to appear in the titled border.
 * @return the newly created Subpanel
 */
//---------------------------------------------------------------------------

protected JPanel createSubPanel (String p_PanelTitle)
    {
    JPanel       l_Panel;
    
    l_Panel = new JPanel ();
    l_Panel.setBorder (new TitledBorder (p_PanelTitle));
    return (l_Panel);    
    }

//---------------------------------------------------------------------------
/**
 * sets the title of the Sub-Panel specified by p_Panel
 * @param p_Panel specifies the Sub-Panel to set the title of
 * @param p_Title specifies the new title
 */
//---------------------------------------------------------------------------

protected void setSubPanelTitle (JPanel p_SubPanel, String p_PanelTitle)
    {
    if (p_SubPanel != null)
        {
        p_SubPanel.setBorder (new TitledBorder (p_PanelTitle));      
        }
    }

//---------------------------------------------------------------------------
/**
 * adds the Sub-Panel specified by p_SubPanel at the bottom of the plugin's layout
 * @param p_SubPanel specifies the sub panel to be added
 */
//---------------------------------------------------------------------------

protected void addSubPanel (JPanel p_SubPanel)
    {
    GridBagLayout       l_Layout;
    GridBagConstraints  l_Constraints;
    Dimension           l_ContentSize;
    Dimension           l_SubPanelSize;
    int                 l_ContentWidth;
    int                 l_ContentHeight;
    
    if (p_SubPanel != null)
        {
        m_Content.remove (m_ContentFiller);
        
        l_ContentSize   = m_Content.getPreferredSize();
        l_SubPanelSize  = p_SubPanel.getPreferredSize();
        
        l_ContentWidth  = (int)l_ContentSize.getWidth();
        l_ContentHeight = (int)(l_ContentSize.getHeight() + l_SubPanelSize.getHeight() + (2 * c_Inset));
        
        l_Layout            = (GridBagLayout)m_Content.getLayout ();
        l_Constraints       = new GridBagConstraints ();
        
        l_Constraints = LayoutToolkit.buildConstraint (0,GridBagConstraints.RELATIVE,1,1,100,1,c_Inset,c_Inset);
        l_Constraints.fill      = GridBagConstraints.HORIZONTAL;
        l_Constraints.anchor	= GridBagConstraints.NORTH;
    
        l_Layout.setConstraints (p_SubPanel,l_Constraints);
        m_Content.add (p_SubPanel);
        
        l_ContentSize = new Dimension (l_ContentWidth, l_ContentHeight);
        m_Content.setPreferredSize (l_ContentSize);
        m_SubPanels.add (p_SubPanel);
        
        l_Constraints = LayoutToolkit.buildConstraint (0,GridBagConstraints.RELATIVE,1,1,100,99,0,0);
        l_Constraints.fill	= GridBagConstraints.BOTH;
        l_Constraints.anchor	= GridBagConstraints.NORTH;
    
        l_Layout.setConstraints (m_ContentFiller,l_Constraints);
        m_Content.add (m_ContentFiller);
        }
    }

//---------------------------------------------------------------------------
/**
 * adds the specified sub panel at the position specified by p_Position. Sub panel
 * previously located at specified position will move down.
 * @param p_SubPanel specifies the sub panel to be added
 * @param p_Position specifies the position to add sub panel at
 */
//---------------------------------------------------------------------------

protected void addSubPanelAt (JPanel p_SubPanel, int p_Position)
    {
    JPanel              l_SubPanel;
    int                 l_SubPanelIndex;

    GridBagLayout       l_Layout;
    GridBagConstraints  l_Constraints;
    Dimension           l_ContentSize;
    Dimension           l_SubPanelSize;
    int                 l_ContentWidth;
    int                 l_ContentHeight;

     
    if (   (p_SubPanel != null)
        && (p_Position >= 0) && (p_Position < m_SubPanels.size()))
        {
        //===================================================================
        // First step consists in removing all Sub-Panels from content panel
        //===================================================================

        m_Content.removeAll ();
                
        //===================================================================
        // Second step consists in adding specified sub-panel at specified 
        // position in sub-panels vector.
        //===================================================================

        m_SubPanels.add (p_Position, p_SubPanel);
        
        l_Layout            = (GridBagLayout)m_Content.getLayout ();
        l_Constraints       = new GridBagConstraints ();
        
        //===================================================================
        // Third step consists in adding all sub-panels from vector to content
        // panel.
        //===================================================================

        for (l_SubPanelIndex = 0; l_SubPanelIndex < m_SubPanels.size(); l_SubPanelIndex++)
            {
            l_SubPanel = (JPanel)m_SubPanels.elementAt (l_SubPanelIndex);

            l_Constraints = LayoutToolkit.buildConstraint (0,GridBagConstraints.RELATIVE,1,1,100,1,c_Inset,c_Inset);
            l_Constraints.fill      = GridBagConstraints.HORIZONTAL;
            l_Constraints.anchor    = GridBagConstraints.NORTH;

            l_Layout.setConstraints (l_SubPanel,l_Constraints);
            m_Content.add (l_SubPanel);
            }
            
        //===================================================================
        // Last step consists in re-adding content filler. Content filler panel
        // is required for proper layout of sub-panels.
        //===================================================================

        l_Constraints = LayoutToolkit.buildConstraint (0,GridBagConstraints.RELATIVE,1,1,100,99,0,0);
        l_Constraints.fill	= GridBagConstraints.BOTH;
        l_Constraints.anchor	= GridBagConstraints.NORTH;
    
        l_Layout.setConstraints (m_ContentFiller,l_Constraints);
        m_Content.add (m_ContentFiller);

        l_ContentSize   = m_Content.getPreferredSize();
        l_SubPanelSize  = p_SubPanel.getPreferredSize();

        l_ContentWidth  = (int)l_ContentSize.getWidth();
        l_ContentHeight = (int)(l_ContentSize.getHeight() + l_SubPanelSize.getHeight() + (2 * c_Inset));
        
        l_ContentSize = new Dimension (l_ContentWidth, l_ContentHeight);
        m_Content.setPreferredSize (l_ContentSize);
        }   
    }

//---------------------------------------------------------------------------

//===========================================================================
//= Plugin Accessors                                                        =
//===========================================================================

//---------------------------------------------------------------------------
/**
 * Returns the Plugin's unique name.
 * @return Plugin unique name
 */
//---------------------------------------------------------------------------

public String getName ()
    {
    return m_Name;
    }

//---------------------------------------------------------------------------
/**
 * Sets the Plugin unique name.
 * @param p_Name specifies the new plugin name
 */
//---------------------------------------------------------------------------

public void setName (String p_Name)
    {
    if (p_Name != null) m_Name = p_Name;
    }

//---------------------------------------------------------------------------
/**
 * Returns the Plugin's icon.
 * @return Plugin icon
 */
//---------------------------------------------------------------------------

public ImageIcon getIcon ()
    {
    return m_Icon;
    }

//---------------------------------------------------------------------------
/**
 * Sets the Plugin icon.
 * @param p_Icon specifies the new plugin icon
 */
//---------------------------------------------------------------------------

public void setIcon (ImageIcon p_Icon)
    {
    if (p_Icon != null) m_Icon = p_Icon;
    }

//---------------------------------------------------------------------------
/**
 * Returns the Plugin's Label.
 * @return Plugin Label
 */
//---------------------------------------------------------------------------

public String getLabel ()
    {
    return m_Label;
    }

//---------------------------------------------------------------------------
/**
 * Sets the Plugin label.
 * @param p_Label specifies the new plugin label
 */
//---------------------------------------------------------------------------

public void setLabel (String p_Label)
    {
    if (p_Label != null)
        {
        m_Label = p_Label;
        m_Title.setText (m_Label);
        }
    }

//---------------------------------------------------------------------------
/**
 * Returns the Plugin's Tooltip.
 * @return Plugin Tooltip
 */
//---------------------------------------------------------------------------

public String getTooltip ()
    {
    return m_Tooltip;
    }

//---------------------------------------------------------------------------
/**
  * Sets the Plugin Tooltip.
 * @param p_Tooltip specifies the new plugin tooltip
 */
//---------------------------------------------------------------------------

public void setTooltip (String p_Tooltip)
    {
    if (p_Tooltip != null) m_Tooltip = p_Tooltip;
    }

//---------------------------------------------------------------------------
/**
 * Returns whether plugin has successfully loaded defaults.
 * @return <B>true</B> if defaults file was successfully loaded, <B>false</B> otherwise.
 */
//---------------------------------------------------------------------------

public boolean hasDefaults ()
    {
    return m_HasDefaults;
    }

//---------------------------------------------------------------------------
/**
 * Returns whether user changed values in plugin form.
 * @return <B>true</B> if user changed values since last update, <B>false</B> otherwise.
 */
//---------------------------------------------------------------------------

public boolean valuesChanged ()
    {
    return m_ValuesChanged;
    }

//---------------------------------------------------------------------------
/**
 * resets the value changed flag to false.
 */
//---------------------------------------------------------------------------

protected void resetValueChanged ()
    {
    m_ValuesChanged = false;
    }

//===========================================================================
//= Primary Plugin interface                                                =
//===========================================================================

//---------------------------------------------------------------------------
/**
 * pluginAdded method will be called automatically AFTER the plugin has been added
 * to the SettingsPanel. Classes required to add Peristence Delegate should
 * overwrite this method to do so.
 */
//---------------------------------------------------------------------------

public void pluginAdded ()
    {
    }

//---------------------------------------------------------------------------
/**
 * the setOwner method will be called by the SettingsPanel upon call to the
 * addPlugin method. The Plugin owner MUST be set before the plugin can access 
 * the centrally stored settings. You don't need to call this method explicitly
 * as it will be called automatically by the owner.
 * @param p_Owner specifies the SettingsPanel this plugin was added to.
 * @param p_PluginID uniquely identifies the plugin among all the plugins owned by the SettingsPanel
 */
//---------------------------------------------------------------------------

public void setOwner (SettingsPanel p_Owner, int p_PluginID) 
    {
    if (p_Owner != null) 
        {
        m_Owner    = p_Owner;
//        m_PluginID = p_PluginID;
        
        pluginAdded ();
        }
    }

//---------------------------------------------------------------------------
/**
 * sets the setting specified by p_Key to the value specified by p_Value. Calling 
 * this method will only have an effect after the plugin was added to the SettingsPanel.
 * @param p_Key specifies the name of the setting to set in the centrally stored settings.
 * @param p_Value specifies the value to set the setting to
 */
//---------------------------------------------------------------------------

protected void setValue (String p_Key, Object p_Value) 
    {
    Object          l_SettingValue;
    boolean         l_ValueChanged;
    SettingEvent    l_Event;
    
    if (m_Owner != null)
        {
        l_SettingValue = getValue (p_Key);
        
        // if value does not exist in config -> put it there
        if (l_SettingValue == null) {
        	l_ValueChanged = true;
        } else {
        	l_ValueChanged = !l_SettingValue.equals (p_Value);
        }
        m_ValuesChanged |= l_ValueChanged;
        m_Owner.setValue (m_Name,p_Key,p_Value);
        
        if (l_ValueChanged)
            {
            l_Event = new SettingEvent (this, m_Name, p_Key);
            m_Owner.fireSettingEvent (l_Event);
            }
        }
    }

//---------------------------------------------------------------------------
/**
 * retrieves the value of the setting specified by p_Key. The method first of all
 * tries to retrieve the value from centrally stored settings. Should this fail, in
 * case the settings file could not be found/loaded, the method tries to get value 
 * from defaults instead. Should this fail to because there are no defaults, the
 * method resorts to retrieving the value from stationary values.
 * @param p_Key specifies the name of the setting to get the value from centrally stored settings.
 * @return value to corresponding to specified setting if exists, NULL otherwise
 */
//---------------------------------------------------------------------------

public Object getValue (String p_Key)
    {
    Object l_Value = null;
    
    if (m_Owner != null) 
        {
        l_Value = m_Owner.getValue (m_Name,p_Key);
        if (l_Value == null) 
            {
            l_Value = getDefault (p_Key);
            if (l_Value == null) l_Value = getStationary (p_Key);
            }
        }
    
    return l_Value;
    }

//---------------------------------------------------------------------------
/**
 * retrieves the default value of the setting specified by p_Key. Calling this method 
 * will only have an effect after defaults were loaded by calling loadDefaults method.
 * @param p_Key specifies the name of the setting to get the default value of.
 * @return default value of specified setting if exists, NULL otherwsie
 */
//---------------------------------------------------------------------------

protected Object getDefault (String p_Key)
    {
    Object l_Default = null;
    
    if (m_Defaults.containsKey(p_Key)) l_Default = m_Defaults.get (p_Key);

    return l_Default;
    }

//---------------------------------------------------------------------------
/**
 * retrieves the stationary value of the setting specified by p_Key. 
 * @param p_Key specifies the name of the setting to get the stationary value of.
 * @return stationary value of specified setting if exists, NULL otherwise
 */
//---------------------------------------------------------------------------

protected Object getStationary (String p_Key)
    {
    Object l_Stationary = null;
    
    if (m_Stationaries.containsKey(p_Key)) l_Stationary = m_Stationaries.get (p_Key);

    return l_Stationary;
    }

//---------------------------------------------------------------------------
/**
 * sets the stationary value for setting specified by p_Key to the value specified by p_Value
 * @param p_Key specifies the setting to set the stationary value for 
 * @param p_Value specifies the stationary value to set
 */
//---------------------------------------------------------------------------

public void setStationary (String p_Key, Object p_Value)
    {
    m_Stationaries.put (p_Key,p_Value);
    }

//---------------------------------------------------------------------------
/**
 * called by the owner upon Apply or Okay action to initiate update of centrally
 * stored settings. Plugins should respond by updating settings so that they reflect
 * the state of plugin controls. Method MUST be overwritten and called by inheriting 
 * plugins.
 */
//---------------------------------------------------------------------------

public void updateSettings ()
    {
    int             l_ExtensionIndex;
    PluginExtension l_Extension;
    
    for (l_ExtensionIndex = 0; l_ExtensionIndex < m_Extensions.size(); l_ExtensionIndex++)
        {
        l_Extension = (PluginExtension)m_Extensions.elementAt (l_ExtensionIndex);
        l_Extension.updateSettings ();
        }
    }

//---------------------------------------------------------------------------
/**
 * called by the owner upon Cancel action to revert to centrally stored
 * settings. Plugins should respond by updating plugin controls the centrally
 * stored values.
 */
//---------------------------------------------------------------------------

public void reflectSettings ()
    {
    int             l_ExtensionIndex;
    PluginExtension l_Extension;
    
    for (l_ExtensionIndex = 0; l_ExtensionIndex < m_Extensions.size(); l_ExtensionIndex++)
        {
        l_Extension = (PluginExtension)m_Extensions.elementAt (l_ExtensionIndex);
        l_Extension.reflectSettings ();
        }
    }

//---------------------------------------------------------------------------
/**
 * called by the owner upon Revert action to revert to default settings.
 * Plugins should respond by updating plugin controls to default values.
  */
//---------------------------------------------------------------------------

public void revertToDefaults ()
    {
    int             l_ExtensionIndex;
    PluginExtension l_Extension;
    
    for (l_ExtensionIndex = 0; l_ExtensionIndex < m_Extensions.size(); l_ExtensionIndex++)
        {
        l_Extension = (PluginExtension)m_Extensions.elementAt (l_ExtensionIndex);
        l_Extension.revertToDefaults ();
        }
    }

//---------------------------------------------------------------------------
/**
 * Method is part of the Relocalizable interface. The method does everything
 * required to reflect changes of active Locale
 */
//---------------------------------------------------------------------------

public void relocalize ()
    {
    int             l_ExtensionIndex;
    PluginExtension l_Extension;

    m_Label     = Translatrix.getTranslationString ("PluginLabel");
    m_Tooltip   = Translatrix.getTranslationString ("PluginTooltip");
    
    for (l_ExtensionIndex = 0; l_ExtensionIndex < m_Extensions.size(); l_ExtensionIndex++)
        {
        l_Extension = (PluginExtension)m_Extensions.elementAt (l_ExtensionIndex);
        l_Extension.relocalize ();
        }
    }

//---------------------------------------------------------------------------
/**
 * adds the Plugin extension specified by p_Extension to the plugin.
 * @param p_Extension specifies the extension to be added to the plugin.
 */
//---------------------------------------------------------------------------

public void addExtension (PluginExtension p_Extension)
    {
    if (p_Extension != null)
        {
        m_Extensions.add (p_Extension);
        p_Extension.setOwner (this);
        
        addSubPanel (p_Extension);
        }
    }

//---------------------------------------------------------------------------
/**
 * adds the Plugin extension specified by p_Extension at the vertical position
 * specified by p_Position.
 * @param p_Extension specifies the extension to be added to the plugin.
 * @param p_Position specifies the vertical position (order) to add specified extension at
 */
//---------------------------------------------------------------------------

public void addExtensionAt (PluginExtension p_Extension, int p_Position)
    {
    if (   (p_Extension != null)
        && (p_Position >= 0)
        && (p_Position < m_SubPanels.size()))
        {
        m_Extensions.add (p_Extension);
        p_Extension.setOwner (this);
    
        addSubPanelAt (p_Extension,p_Position);
        }
    }

//---------------------------------------------------------------------------
/**
 * sets the enabled state of the subpanel specified by p_SubPanelID to the value
 * specified by p_Enabled
 * @param p_SubPanelID specifies the target sub-panel. Top-most sub-panel has ID 0
 * @param p_Enabled specifies whether to enable or disable specified sub-panel
 */
//---------------------------------------------------------------------------

public void setSubPanelEnabled (int p_SubPanelID, boolean p_Enabled)
    {
    JPanel      l_SubPanel;
    int         l_ComponentCount;
    int         l_ComponentID;
    Component   l_Component;
    
    if ((p_SubPanelID >= 0) && (p_SubPanelID < m_SubPanels.size()))
        {
        l_SubPanel = (JPanel)m_SubPanels.elementAt (p_SubPanelID);
        l_ComponentCount = l_SubPanel.getComponentCount ();
        
        for (l_ComponentID = 0; l_ComponentID < l_ComponentCount; l_ComponentID++)
            {
            l_Component = l_SubPanel.getComponent (l_ComponentID);
            l_Component.setEnabled (p_Enabled);
            }
        }
    }

//---------------------------------------------------------------------------
/**
 * adds the Persistence Delegate specified by p_Delegate for class specified by p_Class. 
 * Persistence Delegates take the responsibility for expressing the state of an instance 
 * of a given class. XMLEncoder  class is exclusively designed for the purpose of archiving 
 * graphs of JavaBeans as textual representations of their public properties. Persistence 
 * Delegates are required for storing non JavaBeans compliant classes.
 * @param p_Class specifies the class to add a Persistence Delegate for
 * @param p_Delegate specifies the persistence delegate object to be associated with specifed class
 */
//---------------------------------------------------------------------------

public void addPersistenceDelegate (Class p_Class, PersistenceDelegate p_Delegate)
    {
    if (m_Owner != null) m_Owner.addPersistenceDelegate (p_Class,p_Delegate);
    }

//***************************************************************************
//* End Of Class                                                            *
//***************************************************************************
}
