/*
 * I18nListPlugin.java
 *
 * Created on July 11, 2005, 3:02 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */

package lu.tudor.santec.settings;

import java.io.FileWriter;
import java.io.IOException;
import java.util.logging.Logger;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import lu.tudor.santec.i18n.Translatrix;

import com.jgoodies.forms.builder.PanelBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

/**
 * this class creates start scripts for the java program and offers the
 * possibility to edit the start paramteters for the program and the Java VM.
 * 
 * @author jhermen
 */

public class StartScriptPlugin extends SettingsPlugin implements SettingListener {
	private static final long serialVersionUID = 1L;

	private final static String UNIX_START = "#!/bin/sh";

	private final static String WINDOWS_START = "@echo off";

	private final static String UNIX_CD = "cd $(dirname $0)";

	private final static String UNIX_EXTENSION = ".sh";

	private final static String WINDOWS_EXTENSION = ".cmd";

	private final static String NEWLINE = System.getProperty("line.separator");

	private static final String COMMENT = "created by lu.tudor.santec.settings.StartScriptPlugin (Tudor/Santec)";

	private static final Integer[] memorys = {new Integer(128),new Integer(256),new Integer(512),new Integer(1024),new Integer(2048)};
	
	private static final String VM_ARGS = "";

	private static final String PROG_ARGS = "";

	private String scriptName = "";

	private String jarFile = "";

	private String classPath[] = null;

	private String mainClass = "";

	private FormLayout layout;

	private JPanel scriptPanel;

	private JLabel desc;

	private JLabel vmArgsLabel;

	private JTextArea vmArgs;

	private JTextArea progArgs;

	private JLabel progArgsLabel;

	private JLabel memoryLabel;

	private JComboBox memory;
	
	private static Logger logger = Logger
	.getLogger("lu.tudor.santec.settings.StartScriptPlugin");

	/**
	 * creates a new StartScriptPanel with the following parameters
	 * 
	 * @param p_Name
	 *            the name of the panel
	 * @param scriptName
	 *            the name of the Startscripts without file extension
	 * @param jarFile
	 *            the jarFile to start
	 */
	public StartScriptPlugin(String p_Name, String scriptName, String jarFile) {
		super(p_Name);
		this.scriptName = scriptName;
		this.jarFile = jarFile;
		buildPanel();
		relocalize();
	}

	/**
	 * creates a new StartScriptPanel with the following parameters
	 * 
	 * @param p_Name
	 *            the name of the panel
	 * @param scriptName
	 *            the name of the Startscripts without file extension
	 * @param classPath
	 *            the classpath of the application
	 * @param mainClass
	 *            the mainclass of the application
	 */
	public StartScriptPlugin(String p_Name, String scriptName,
			String classPath[], String mainClass) {
		super(p_Name);
		this.scriptName = scriptName;
		this.classPath = classPath;
		this.mainClass = mainClass;
		buildPanel();
		relocalize();
	}

	/**
	 * creates the Panel
	 */
	private void buildPanel() {

		setLabel(Translatrix.getTranslationString("script:Label"));
		setStationary("Memory", new Integer(256));
		setStationary("VmArgs", VM_ARGS);
		setStationary("ProgArgs", PROG_ARGS);

		this.layout = new FormLayout("right:50dlu,5dlu,fill:60dlu:grow,5dlu",
				"center:pref,5dlu,center:pref,5dlu,pref,5dlu,40dlu,5dlu,pref,5dlu,40dlu");
		CellConstraints cc = new CellConstraints();

		this.scriptPanel = createSubPanel(Translatrix
				.getTranslationString("lnf:Title"));
		this.scriptPanel.setLayout(layout);

		PanelBuilder pBuilder = new PanelBuilder(layout, scriptPanel);

		this.desc = new JLabel();

		pBuilder.add(desc, cc.xyw(1, 1, 3, "fill, center"));

		this.memoryLabel = new JLabel();

		pBuilder.add(memoryLabel, cc.xy(1, 3));

		this.memory = new JComboBox(memorys);

		pBuilder.add(this.memory, cc.xy(3, 3));
		
		this.vmArgsLabel = new JLabel();

		pBuilder.add(vmArgsLabel, cc.xy(1, 5));

		this.vmArgs = new JTextArea();

		pBuilder.add(new JScrollPane(this.vmArgs), cc.xywh(3, 5, 1, 3));

		this.progArgsLabel = new JLabel();

		pBuilder.add(progArgsLabel, cc.xy(1, 9));

		this.progArgs = new JTextArea();

		pBuilder.add(new JScrollPane(this.progArgs), cc.xywh(3, 9, 1, 3));

		addSubPanel(scriptPanel);

		setIcon(getIconResource("icn_scripts"));
		
		setLabel(Translatrix.getTranslationString("script:Label"));
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see lu.tudor.santec.settings.SettingsPlugin#revertToDefaults()
	 */
	public void revertToDefaults() {

		this.memory.setSelectedItem(getDefault("Memory"));
		this.vmArgs.setText((String) getDefault("VmArgs"));
		this.progArgs.setText((String) getDefault("ProgArgs"));

		reflectSettings();
		super.revertToDefaults();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see lu.tudor.santec.settings.SettingsPlugin#updateSettings()
	 */
	public void updateSettings() {
		setValue("Memory", (Integer)this.memory.getSelectedItem());
		setValue("VmArgs", this.vmArgs.getText());
		setValue("ProgArgs", this.progArgs.getText());
		super.updateSettings();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see lu.tudor.santec.settings.SettingsPlugin#reflectSettings()
	 */
	public void reflectSettings() {
		super.reflectSettings();
		this.memory.setSelectedItem(getValue("Memory"));
		this.vmArgs.setText((String) getValue("VmArgs"));
		this.progArgs.setText((String) getValue("ProgArgs"));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see lu.tudor.santec.i18n.Relocalizable#relocalize()
	 */
	public void relocalize() {
		setLabel(Translatrix.getTranslationString("script:Label"));
		setSubPanelTitle(scriptPanel, Translatrix
				.getTranslationString("script:Title"));
		this.desc.setText(Translatrix
				.getTranslationString("script:Description"));
		this.memoryLabel.setText(Translatrix
				.getTranslationString("script:Memory"));
		this.vmArgsLabel.setText(Translatrix
				.getTranslationString("script:VMLabel"));
		this.progArgsLabel.setText(Translatrix
				.getTranslationString("script:ProgLabel"));
	}

	/**
	 * creates a unix compatible startup shell script
	 */
	public void createUnixScript() {
		try {
			FileWriter fw = new FileWriter(this.scriptName + UNIX_EXTENSION);

			fw.write(UNIX_START + NEWLINE);

			fw.write("# " + COMMENT + NEWLINE);

			fw.write(UNIX_CD + NEWLINE);

			fw.write("java ");
			fw.write("-Xmx"+this.memory.getSelectedItem()+ "M ");
			fw.write(this.vmArgs.getText().replaceAll(NEWLINE, " ") + " ");
			if (this.jarFile != null && this.jarFile != "") {
				fw.write("-jar " + this.jarFile + " ");
			} else {
				fw.write("-cp " + createClasspath(this.classPath, ":") + " ");
				fw.write(this.mainClass + " ");
			}
			fw.write(this.progArgs.getText().replaceAll(NEWLINE, " ") + " ");

			fw.close();

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * creates a windows compatible startup cmd script
	 */
	public void createWindowsScript() {
		try {
			FileWriter fw = new FileWriter(this.scriptName + WINDOWS_EXTENSION);

			fw.write(WINDOWS_START + NEWLINE);

			fw.write("rem " + COMMENT + NEWLINE);

			fw.write("start javaw ");
			fw.write("-Xmx"+this.memory.getSelectedItem()+ "M ");
			fw.write(this.vmArgs.getText().replaceAll(NEWLINE, " ") + " ");
			if (this.jarFile != null && this.jarFile != "") {
				fw.write("-jar " + this.jarFile + " ");
			} else {
				fw.write("-cp " + createClasspath(this.classPath, ";") + " ");
				fw.write(this.mainClass + " ");
			}
			fw.write(this.progArgs.getText().replaceAll(NEWLINE, " ") + " ");

			fw.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * creates the classpath with the system dependent delimiter
	 * 
	 * @param classPath
	 *            all files in the classpath
	 * @param seperator
	 *            the sepetator (:|;)
	 * @return the classpath as String
	 */
	private String createClasspath(String[] classPath, String seperator) {
		String path = "";
		for (int i = 0; i < classPath.length; i++) {
			path += classPath[i] + seperator;
		}
		return path;
	}
	
	/* (non-Javadoc)
	 * @see lu.tudor.santec.settings.SettingListener#settingChanged(lu.tudor.santec.settings.SettingEvent)
	 */
	public void settingChanged(SettingEvent p_Event) {
		if (p_Event.getPluginName().equals(this.getName())) {
			this.createWindowsScript();
			this.createUnixScript();
			logger.info("Created new startup scripts: "+ this.scriptName + UNIX_EXTENSION + ", " + this.scriptName + WINDOWS_EXTENSION);
		}
	}
	
	/* (non-Javadoc)
	 * @see lu.tudor.santec.settings.SettingsPlugin#pluginAdded()
	 */
	public void pluginAdded() {
		m_Owner.addSettingListener(this);
	}

}
